@@ -4,10 +4,12 @@ import hmac
 from requests import Session, Response
 from typing import Callable, Union
 from instauto.api.actions.stubs import _request
-from dataclasses import asdict
 
-from ..structs import Method, State, DeviceProfile, IGProfile
-from .structs.post import Post, Comment, UpdateCaption, Save, Like, Unlike, Device, RetrieveByUser, Location
+
+from ..structs import Method, State, DeviceProfile, IGProfile, PostLocation
+from .structs.post import PostFeed, PostStory, Comment, UpdateCaption, Save, Like, Unlike, Device, RetrieveByUser, \
+    Location, RetrieveByTag, RetrieveLikers, RetrieveCommenters
+
 from ..exceptions import BadResponse
 
 from .helpers import build_default_rupload_params
@@ -26,13 +28,13 @@ class PostMixin:
     breadcrumb_private_key: bytes
     bc_hmac: hmac.HMAC
 
-    def _post_act(self, obj: Union[Save, Post, Comment, UpdateCaption, Like, Unlike]):
+    def _post_act(self, obj: Union[Save, Comment, UpdateCaption, Like, Unlike]):
         """Peforms the actual action and calls the Instagram API with the data provided."""
         if obj.feed_position is None:
             delattr(obj, 'feed_position')
 
         endpoint = f'media/{obj.media_id}/{obj.action}/'
-        return self._request(endpoint, Method.POST, data=obj.__dict__, signed=True)
+        return self._request(endpoint, Method.POST, data=obj.fill(self).to_dict(), signed=True)
 
     def post_like(self, obj: Like) -> Response:
         """Likes a post"""
@@ -77,7 +79,36 @@ class PostMixin:
 
         return str(as_json['venues'][0]['external_id'])
 
-    def post_post(self, obj: Post, quality: int = None) -> Response:
+    def _upload_image(self, obj: Union[PostStory, PostFeed], quality: int) -> dict:
+        as_dict = obj.fill(self).to_dict()
+        headers = {
+            'x-fb-photo-waterfall-id': str(as_dict.pop('x_fb_waterfall_id')),
+            'x-entity-length': str(as_dict.pop('entity_length')),
+            'x-entity-name': as_dict.pop('entity_name'),
+            'x-instagram-rupload-params': json.dumps(build_default_rupload_params(obj, quality)),
+            'x-entity-type': as_dict.pop('entity_type'),
+            'offset': '0',
+            'scene_capture_type': 'standard',
+            'creation_logger_session_id': self.state.session_id
+        }
+
+        path = as_dict.pop('image_path')
+        if hasattr(obj, 'location') and obj.location is not None:
+            if not obj.location.facebook_places_id:
+                obj.location.facebook_places_id = self._request_fb_places_id(obj.location)
+            as_dict['location'] = json.dumps(obj.location.__dict__)
+
+        if obj.device is None:
+            d = Device(self.device_profile.manufacturer, self.device_profile.model,
+                       int(self.device_profile.android_sdk_version), self.device_profile.android_release)
+            obj.device = d
+
+        with open(path, 'rb') as f:
+            self._request(f'https://i.instagram.com/rupload_igphoto/{headers["x-entity-name"]}', Method.POST,
+                          headers=headers, data=f.read())
+        return as_dict
+
+    def post_post(self, obj: Union[PostStory, PostFeed], quality: int = None) -> Response:
         """Uploads a new picture/video to your Instagram account.
         Parameters
         ----------
@@ -90,49 +121,18 @@ class PostMixin:
         Response
             The response returned by the Instagram API.
         """
-
         if quality is None:
             quality = 70
-
-        # sets all data of the PostPost object that is stored in the ig_profile, device_profile or state attributes.
-        if obj.device is None:
-            d = Device(self.device_profile.manufacturer, self.device_profile.model,
-                       int(self.device_profile.android_sdk_version), self.device_profile.android_release)
-            obj.device = d
-
-        as_dict = obj.to_dict()
-        # Instagram will refuse the request and respond with a 400 bad request if an empty location is send along
-        if as_dict['location'] is None: as_dict.pop('location')
-
-        rupload_params = build_default_rupload_params(obj, quality)
-
-        # headers used when uploading the object to Instagram
-        headers = {
-            'x-fb-photo-waterfall-id': str(as_dict.pop('x_fb_waterfall_id')),
-            'x-entity-length': str(as_dict.pop('entity_length')),
-            'x-entity-name': as_dict.pop('entity_name'),
-            'x-instagram-rupload-params': json.dumps(rupload_params),
-            'x-entity-type': as_dict.pop('entity_type'),
-            'offset': '0',
-            'scene_capture_type': 'standard',
-            'creation_logger_session_id': self.state.session_id
-        }
-
-        path = as_dict.pop('image_path')
-        if obj.location is not None:
-            if not obj.location.facebook_places_id:
-                obj.location.facebook_places_id = self._request_fb_places_id(obj.location)
-            as_dict['location'] = json.dumps(obj.location.__dict__)
-
-        # upload the image to Instagram
-        with open(path, 'rb') as f:
-            self._request(f'https://i.instagram.com/rupload_igphoto/{headers["x-entity-name"]}', Method.POST,
-                          headers=headers, data=f.read())
-        # headers used for configuring the object just uploaded
+        as_dict = self._upload_image(obj, quality)
         headers = {
             'retry_context': json.dumps({"num_reupload": 0, "num_step_auto_retry": 0, "num_step_manual_retry": 0})
         }
-        return self._request('media/configure/', Method.POST, data=as_dict, headers=headers, signed=True)
+        if obj.source_type == PostLocation.Feed.value:
+            return self._request('media/configure/', Method.POST, data=as_dict, headers=headers, signed=True)
+        elif obj.source_type == PostLocation.Story.value:
+            return self._request('media/configure_to_story/', Method.POST, data=as_dict, headers=headers, signed=True)
+        else:
+            raise Exception("{} is not a supported post location.", obj.source_type)
 
     def post_retrieve_by_user(self, obj: RetrieveByUser) -> (RetrieveByUser, Union[dict, bool]):
         """Retrieves 12 posts of the user at a time. If there was a response / if there were any more posts
@@ -149,7 +149,8 @@ class PostMixin:
         if obj.page > 0 and obj.max_id is None:
             return obj, False
 
-        as_dict.pop('max_id')
+        if obj.max_id is not None:
+            as_dict.pop('max_id')
         as_dict.pop('user_id')
 
         resp = self._request(f'feed/user/{obj.user_id}/', Method.GET, query=as_dict)
@@ -158,3 +159,31 @@ class PostMixin:
         obj.max_id = resp_as_json.get('next_max_id')
         obj.page += 1
         return obj, resp_as_json['items']
+
+    def post_retrieve_by_tag(self, obj: RetrieveByTag) -> (RetrieveByTag, Union[dict, bool]):
+        as_dict = obj.to_dict()
+
+        if obj.page > 0 and obj.max_id is None:
+            return obj, False
+
+        as_dict.pop('tag_name')
+
+        resp = self._request(f'feed/tag/{obj.tag_name}/', Method.GET, query=as_dict)
+        resp_as_json = resp.json()
+
+        obj.max_id = resp_as_json.get('next_max_id')
+        obj.page += 1
+        return obj, resp_as_json['items']
+
+    def post_get_likers(self, obj: RetrieveLikers) -> [any]:
+        """Retrieve all likers of specific media_id"""
+        endpoint = 'media/{media_id}/likers'.format(media_id=obj.media_id)
+        resp = self._request(endpoint=endpoint, method=Method.GET)
+        users_as_json = resp.json().get('users')
+        return users_as_json
+      
+    def post_get_commenters(self, obj: RetrieveCommenters) -> [any]:
+        endpoint = 'media/{media_id}/comments'.format(media_id=obj.media_id)
+        resp = self._request(endpoint=endpoint, method=Method.GET)
+        users_as_json = resp.json().get('users')
+        return users_as_json
