@@ -10,5 +10,14 @@ class Username(cmmn.Base):
     def __init__(self, q: str, count: int, *args, **kwargs):
         self.q = q
         self.count = count
-        self.timezone_offset = str(time.localtime().tm_gtoff)
+        self.timezone_offset = str(time.localtime().tm_gmtoff)
+        super().__init__(*args, **kwargs)
+
+class Tag(cmmn.Base):
+    q: str
+    count: int
+
+    def __init__(self, q: str, count: int, *args, **kwargs):
+        self.q = q
+        self.count = count
         super().__init__(*args, **kwargs)
