@@ -13,13 +13,20 @@ from .structs import IGProfile, DeviceProfile, State
 from .constants import (DEFAULT_IG_PROFILE, DEFAULT_DEVICE_PROFILE, DEFAULT_STATE)
 from .exceptions import StateExpired, NoAuthDetailsProvided, CorruptedSaveData
 
-from .actions import AuthenticationMixIn, PostMixin, RequestMixIn, ProfileMixin, FriendshipsMixin, SearchMixin
+from .actions.profile import ProfileMixin
+from .actions.authentication import AuthenticationMixin
+from .actions.post import PostMixin
+from .actions.request import RequestMixin
+from .actions.friendships import FriendshipsMixin
+from .actions.search import SearchMixin
+from .actions.challenge import ChallengeMixin
+from .actions.direct import DirectMixin
 
 logger = logging.getLogger(__name__)
 logging.captureWarnings(True)
 
 
-class ApiClient(PostMixin, AuthenticationMixIn, RequestMixIn, ProfileMixin, FriendshipsMixin, SearchMixin):
+class ApiClient(ProfileMixin, AuthenticationMixin, PostMixin, RequestMixin, FriendshipsMixin, SearchMixin, ChallengeMixin, DirectMixin):
     """Class used to access all features, in an ideal situation this is the only class that needs to be imported (
     along with the structs)"""
     breadcrumb_private_key = "iN4$aGr0m".encode()
